#version 430


in vec3 normalInterp;
in vec3 vertPos;

uniform sampler2D colorTexture;
uniform sampler2DShadow shadowMap;
//uniform sampler2D shadowMap;
in vec4 fragPosLightSpace;

uniform vec3 material;
uniform vec3 viewPos;
const float shininess = 1.0;
const float specular = 1.0;

in vec3 worldNormal;
in vec2 fUVs;

in float alpha_out;
uniform vec3 lightDirection;
uniform vec3 lightPosition;
uniform float lightCut;
uniform float lightOuterCut;
const vec3 ambient = vec3(0.4, 0.4, 0.4);
const vec3 diffuse = vec3(0.9, 0.9, 0.9);
const vec3 specColor = vec3(1.0, 1.0, 1.0);
float constant = 1.0;
float linear = 0.09;
float quadratic = 0.032;
layout(location = 0) out vec4 fragColor;

void main() { 
	
	vec3 texcolor = texture(colorTexture,fUVs).rgb;
	vec3 norm = normalize(normalInterp);
	vec3 lightDir = normalize(lightPosition - vertPos);

	// spotlight cutoff
	float theta = dot(lightDir, normalize(-lightDirection)); 
    float epsilon = (lightCut - lightOuterCut);
    float intensity = clamp((theta - lightOuterCut) / epsilon, 0.0, 1.0);

	// Shadow
	float visibility = textureProj(shadowMap, fragPosLightSpace); 

	// Ambient
	vec3 ambient = ambient *texcolor;
  	
	// Diffuse 
	float diff = max(dot(norm, lightDir), 0.0);
	vec3 diffuse = diffuse * diff * texcolor;  
    
	// Specular
	vec3 viewDir = normalize(viewPos - vertPos);
	vec3 reflectDir = reflect(-lightDir, norm);  
	float spec = pow(max(dot(viewDir, reflectDir), 0.0), shininess);
	vec3 specular = specular * spec * texcolor;
    
	// Attenuation
	float distance    = length(lightDirection - vertPos);
	float attenuation = 1.0f / (constant + linear * distance + quadratic * (distance * distance));    



	ambient  *= attenuation;  
	diffuse  *= attenuation * visibility * intensity;
	specular *= attenuation * visibility * intensity; 


	fragColor = vec4((ambient + diffuse + specular) * texcolor, alpha_out);   
}